import subprocess
import os


def run():
    ui = _run_flask()
    print(f'UI started: PID {ui.pid}')
    _run_simulator()


def run_debug():
    ui = _run_flask_debug()
    print(f'UI started: PID {ui.pid}')
    _run_simulator('debug')


def _run_flask_debug() -> subprocess.Popen:
    env_ui = {
        'FLASK_APP': 'ui',
        'FLASK_ENV': 'development',
        'FLASK_DEBUG': '1',
        'FLASK_RUN_PORT': '2009'
    }
    ui = subprocess.Popen(['python3', '-m', 'flask', 'run'],
                          env={
                              **os.environ.copy(),
                              **env_ui
                          },
                          universal_newlines=True)
    return ui


def _run_flask() -> subprocess.Popen:
    env_ui = {
        'FLASK_APP': 'ui',
        'FLASK_ENV': 'production',
        'FLASK_DEBUG': '0',
        'FLASK_RUN_PORT': '2009'
    }
    ui = subprocess.Popen(['python3', '-m', 'flask', 'run'],
                          env={
                              **os.environ.copy(),
                              **env_ui
                          },
                          universal_newlines=True,
                          stdout=subprocess.DEVNULL,
                          stderr=subprocess.DEVNULL)
    return ui


def _run_simulator(mode: str = 'info'):
    import modbus.simulator as sim
    simulator_path = os.path.abspath(sim.__file__)
    simulator = subprocess.Popen(
        ['python3', simulator_path, '--log', mode],
        universal_newlines=True,
    )
    print(f'MODBUS simulator started: PID {simulator.pid}')
    try:
        simulator.wait()
    except KeyboardInterrupt:
        simulator.wait()
